// (C) 2023 by Folkert van Heusden <mail@vanheusden.com>
// CC0 license

#define INPUT_ANALOGUE A0

#define PIN_A0 12
#define PIN_NOT_WR 13
#define PIN_NOT_DTACK 10
#define PIN_CLOCK 11

// #define SOUND_EFFECT

void set_register(const byte address, const byte data){
	digitalWrite(PIN_A0, HIGH);

	for(int i=0; i<8; i++)
		digitalWrite(2 + i, !!(address & (1 << i)));

	bool ack = digitalRead(PIN_NOT_DTACK);

	digitalWrite(PIN_NOT_WR, LOW);
	delayMicroseconds(5);
	digitalWrite(PIN_NOT_WR, HIGH);

	while(ack)
		ack = digitalRead(PIN_NOT_DTACK);

	delayMicroseconds(5);

	digitalWrite(PIN_A0, LOW);

	for(int i=0; i<8; i++)
		digitalWrite(2 + i, !!(data & (1 << i)));

	ack = digitalRead(PIN_NOT_DTACK);

	digitalWrite(PIN_NOT_WR, LOW);
	delayMicroseconds(5);
	digitalWrite(PIN_NOT_WR, HIGH);

	while(ack)
		ack = digitalRead(PIN_NOT_DTACK);
}

void mydelay(unsigned long duration) {
	unsigned long start = millis();
	unsigned long end   = start + duration;

	do {
		uint16_t v = analogRead(INPUT_ANALOGUE);

		OCR2A = max(1, v >> 2);
	}
	while(millis() < end);
}

void setup() {
	Serial.begin(115200);

	// 8MHz TTL out on pin 11
	pinMode(PIN_CLOCK, OUTPUT);

	TCCR2A = (1 << WGM21) | (1 << COM2A0);
	TCCR2B = 1 << CS20;
	TIMSK2 = 0;
	OCR2A = 1;  // 1 = 4MHz
	//

	pinMode(PIN_NOT_WR, OUTPUT);  // /WR

	pinMode(PIN_A0, OUTPUT);  // A0 (control / address sel.)

	pinMode(PIN_NOT_DTACK, INPUT);  // /DTACK

	for(int i=2; i<10; i++)
		pinMode(i, OUTPUT);

#ifdef SOUND_EFFECT
	for(int i=0; i<6; i++)  // set volume
		set_register(i, 255);

	for(int i=0; i<6; i++)  // set frequency
		set_register(8 + i, 100);

	for(int i=0; i<6; i++)  // set octave
		set_register(0x10 + i, 4);
#else
	set_register(0, 255);  // volume

	set_register(8, 100);  // frequency

	set_register(0x10, 4); // set octave
#endif

	set_register(0x14, 63);  // frequency enable

	set_register(0x15, 0);  // disable noise

	set_register(0x18, 11);
	set_register(0x18, 139);

	set_register(0x1c, 1);  // reset frequencies / enable sound
}

void loop() {
#ifdef SOUND_EFFECT
	for(int x = 255; x > 0; x -= 2) {
		for(int i=0; i<6; i++)
			set_register(0x10 + i, x);

		mydelay(50);
	}
#else
	mydelay(1000);
#endif
}
